<?php

//-------------------------------------------------------------
//
// $Id: class.mutant_forum_tree.php,v 1.3 2004/04/05 01:25:10 kuatodotorg Exp $
//
// STARTED   : 02/04/2004
// COPYRIGHT : 2002-2004 kuato.org
// WWW       : http://www.kuato.org/
// LICENSE   : GPL vs2.0 [ see /docs/COPYING ] 
//
// DESCRIPTION:
// Class to support a forum tree needed for many parts of our portal.
// Since phpbb 2.2 supports unlimited forums we need this class for fetching
// a forum and all its children and to provide methods of operating on the data.
//              
//-------------------------------------------------------------

class MutantForumTree
{
	/** 
	* Forum array to represent our tree
	*
	* @var      array	Associate array representing forum table rows in the database
	* @access   private
	*/
	var $_forums = array();

	/** 
	* Database object
	*
	* @var      object	A reference to a phpbb database object
	* @access   private
	*/
	var $_db;

	/**
	* Constructor
	*
	* Fetch the forum_id and all its children into an array
	*
	* @param object		$db			Reference to the database access object
	* @param int		$forum_id	The parent forum id 	
	*
	* @access   public
	* @return   void 
	*/
	function MutantForumTree(&$db, $forum_id)
	{
		$this->_db =& $db;

		// Get the left and right ids
		$sql = 'SELECT
					f.left_id, f.right_id 
				FROM 
					'.FORUMS_TABLE. " f 
				WHERE
					f.forum_id = $forum_id";
		$result = $this->_db->sql_query($sql);

		if( ($row = $this->_db->sql_fetchrow($result)) )
		{	
			$this->_db->sql_freeresult($result);

			// Collect the forum tree for the parent and all its children in one 
			// fell swoop thanks to MPTT
			$sql = 'SELECT 
						f.forum_id, f.left_id, f.right_id, f.forum_name, f.forum_image, 
						f.forum_desc, f.parent_id, f.forum_parents, f.forum_topics
					FROM 
						'.FORUMS_TABLE. ' f 
					WHERE f.left_id BETWEEN '. $row['left_id']. ' AND '. $row['right_id']. ' 
					ORDER BY f.left_id ASC';
			$result = $this->_db->sql_query($sql);

			if( ($this->_forums = $this->_db->sql_fetchrowset($result)) )
			{
				$this->_db->sql_freeresult($result);
			}
		}
	}

	/** 
	* Create a list of the forums from the array of rows and return
	* a reference to it.
	*
	* @param int $root_id	Root id of the forum
	* @param int $cid		Current forum id
	*
	* @access	public
	* @return	array
	*/
	function &createForumList($root_id, $cid)
	{
		global $auth, $user;

		$forum_list = array();
		$num_rows = count($this->_forums); 

		for( $i = 0; $i < $num_rows; ++$i )
		{
			// Skip if not authorized to view forum
			if( !$auth->acl_get('f_list', $this->_forums[$i]['forum_id']) )
			{
				continue;
			}
			
			// If current forum is the root of all news then collect 
			// itself and all its children
			if( $cid == $root_id )
			{
				$forum_list[] = $this->_forums[$i]['forum_id'];
			}
			else
			{
				// Otherwise just the current forum and its children
				if( ($this->_forums[$i]['forum_id'] == $cid) || ($this->_forums[$i]['parent_id'] == $cid) )
				{
					$forum_list[] = $this->_forums[$i]['forum_id'];
				}
			}
		}

		return $forum_list;
	}

	/** 
	* Accessor
	*
	* Return a reference to our forum array
	*
	* @access	public
	* @return	array
	*/
	function &getForums()
	{
		return $this->_forums;
	}

	/** 
	* Compute an array offset
	*
	* @access	private
	* @return	int
	*/
	function _array_offset($array, $offset_key)
	{
		$offset = 0;

		foreach( $array as $key => $val )
		{
			if( $key == $offset_key )
				return $offset;
			$offset++;
		}       
			
		return -1;
	}

	/** 
	* Create the navigation links starting at root_id 
	* and ending at forum_id.
	*
	* @param int 	$root_id	Root forum
	* @param int	$forum_id	Current forum
	*
	* @access	public
	* @return	string
	*/
	function createNavLinks($root_id, $forum_id)
	{
		// Find forum_id in forum_row and save ref to that row
		$nbr = count($this->_forums);
		for( $i = 0; $i < $nbr; $i++ )
		{
			if( $this->_forums[$i]['forum_id'] == $forum_id )
			{
				$forum =& $this->_forums[$i];
				break;
			}
		}

		// Fetch parents array of the forum and unserialize
		$parents = unserialize($forum['forum_parents']);

		// Traverse parents array creating new array starting at root_id
		$offset = $this->_array_offset($parents, $root_id);
		$new = ( $offset != -1 ) ? array_slice($parents, $offset) : array();

		// Append the forum name
		$new[] = $forum['forum_name'];
		$new_str = implode(" &#8250&#8250 ", $new);

		// Done finally!
		return $new_str;
	}
}

